import os
import matplotlib.pyplot as plt
from photosorganisation.get_total_size import get_size
import sys
from photosorganisation.duplicates import find_duplicates, move_duplicates
###
def get_photos(file_path):
    ListOfImages = []
    sub_dirs = []
    dirs = []
    for subdir, dirs, files in os.walk(file_path):
        sub_dirs.append(subdir)
        if "Duplicates" in dirs:
            dirs.remove("Duplicates")
        if "Blurry" in dirs:
            dirs.remove("Blurry")
        if "Similar" in dirs:
            dirs.remove("Similar")
        if "Memes" in dirs:
            dirs.remove("Memes")
        if "Screenshots" in dirs:
            dirs.remove("Screenshots")
        for file in files:
            if file.lower().endswith(('.png', '.jpg', '.jpeg', '.tiff', '.bmp', '.gif')):
                ListOfImages.append(os.path.join(subdir, file))
    return ListOfImages,sub_dirs



if __name__ == "__main__":
    folder_path = sys.argv[1]

    total_size = get_size(folder_path)
    received_photos,folders = get_photos(folder_path)
    total_photos = len(received_photos)
    total_folders = len(folders)-1
    print(f"Found total {total_photos} photos from {total_folders} sub-folders. Total size: {total_size}")
    #print(received_photos)
    duplicates,hash_keys = find_duplicates(received_photos)
    print(f"Found {len(duplicates)} duplicates")
    move_duplicates(duplicates,folder_path)
   # total_duplicates_size =

#print((get_photos('/home/amulyamantha/code/jaseppala/photosorganisation/raw_data/')[1]))



"""Imports"""
from photosorganisation.photos import get_photos
from hashlib import md5
import PIL
from imageio import imread
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
import time
import numpy as np
import cv2
import os
import scipy
import itertools
from  scipy.spatial.distance import hamming
from photosorganisation.get_total_size import get_size
import sys
from photosorganisation.duplicates import find_duplicates, move_duplicates
import shutil
from termcolor import colored
import emoji

#
def img_gray(image):
    image = imread(image)
    return np.average(image,axis=2,weights=[0.2999,0.587,0.114])
#
#resize image and flatten
def resize(image, height=30, width=30):
    row_res = cv2.resize(image,(height, width), interpolation = cv2.INTER_AREA).flatten()
    col_res = cv2.resize(image,(height, width), interpolation = cv2.INTER_AREA).flatten('F')
    return row_res, col_res
#
#gradient direction based on intensity

def intensity_diff(row_res, col_res):
    difference_row = np.diff(row_res)
    difference_col = np.diff(col_res)
    difference_row = difference_row > 0
    difference_col = difference_col > 0
    return np.vstack((difference_row, difference_col)).flatten()
#
def file_hash(array):
    return md5(array).hexdigest()
#


def difference_score(image, height=30, width=30):
    gray = img_gray(image)
    row_res, col_res = resize(gray, height, width)
    difference = intensity_diff(row_res, col_res)

    return difference
#
def hamming_distance(image, image2):
    score =scipy.spatial.distance.hamming(image, image2)
    return score
#
def difference_score_dict_hem(image_list):
    ds_dict = {}
    duplicates = []
    for image in image_list:
        ds = difference_score(image)

        if image not in ds_dict:
            ds_dict[image] = ds
        else:
            duplicates.append((image, ds_dict[image]))

    return duplicates, ds_dict
##
def move_similar(similarList,folder_path):
    similar_folder = os.path.join(folder_path, 'Similar')
    if not os.path.exists(similar_folder):
        os.mkdir(similar_folder)
    if len(similarList) > 0:
        for file in similarList:
            #print(file)
            shutil.move(file, similar_folder)
            total_Similar_size = get_size(os.path.join(folder_path, 'Similar'))
        print(emoji.emojize(colored(f"Found near duplicates. Delete unnecessary!! Save Space!! :hourglass_not_done:", "red")))
        print(emoji.emojize(colored(f"Found and moved {len(similarList)} Photos which are similar, Folder size:{total_Similar_size} :man_dancing:","green")))

####
def sort_similar(received_photos):
    similar_lists = []
    similar, ds_dict_hem =difference_score_dict_hem(received_photos)
    for k1,k2 in itertools.combinations(ds_dict_hem, 2):
        if hamming_distance(ds_dict_hem[k1], ds_dict_hem[k2])< .31:
            similar_lists.append(k1)#
            similar_lists.append(k2)
    return set(similar_lists)

if __name__ == "__main__":
    folder_path = sys.argv[1]
    received_photos, folders = get_photos(folder_path)
    similar = sort_similar(received_photos)
    move_similar(similar, folder_path)

    print(colored(f"Found and moved {len(similar)} Photos similar to one another", "yellow"))



# imports
import hashlib
import PIL
from imageio import imread
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
import time
import numpy as np
from hashlib import md5
import os
import shutil
from termcolor import colored
from photosorganisation.get_total_size import get_size
import emoji

def find_duplicates(filepath):
    duplicates = []
    hash_keys = dict()
    print(emoji.emojize(colored(" :eyes: Looking for Duplicates..", "cyan", "on_white",attrs=['bold'])))
    for index, filename in enumerate(filepath):
        #print(filename)
        if os.path.isfile(filename):
            with open(filename,'rb') as  f:
                #print(filename)
                filehash = hashlib.md5(f.read()).hexdigest()
                #print(filehash)
            if filehash not in hash_keys:
                hash_keys[filehash] = index
            else:
                #print("Duplicate found")
                duplicates.append((filename,index,hash_keys[filehash]))
    return duplicates, hash_keys
#
def move_duplicates(duplicatesList,folder_path):
    if len(duplicatesList) > 0:
        duplicates_folder = os.path.join(folder_path, 'Duplicates')
        #print(duplicates_folder)
        if not os.path.exists(duplicates_folder):
            os.mkdir(duplicates_folder)
        for file in duplicatesList:
            #print(file)
            shutil.move(file[0], duplicates_folder)
        total_duplicates_size = get_size(os.path.join(folder_path, 'Duplicates'))
        print(colored(f"Moved Duplicates to folder, size: {total_duplicates_size}","green"))



import os
from os.path import isdir
import cv2
import numpy as np
from google.cloud import storage
from zipfile import ZipFile

from tensorflow.keras.applications.vgg16 import preprocess_input

def get_data_gcp(GCP_FILE_NAME):
    """downloads a zip file into the current working directory and extracts its contents into the raw data folder

    Args:
        GCP_FILE_NAME ([string]): [Name of the zip file]
    """
    client = storage.Client(project = 'le-wagon-ds-bootcamp-318909')
    bucket = client.get_bucket('lewagon-photo-classifier')
    blob = bucket.get_blob(GCP_FILE_NAME)
    blob.download_to_filename(os.path.join(os.getcwd(), GCP_FILE_NAME))

    with ZipFile(os.path.join(os.getcwd(), GCP_FILE_NAME), 'r') as zipObj:

        zipObj.extractall('../raw_data')

def load_data(path, how = 'one', grayscale = True, size = (100,100), asarray = True, n_img = 'all'):
    """loads all images into an array

    path: path to the folder in which the images or folders full of images are
    how: 'one' to load files from one folder, 'many' if images are in subfolders (default 'one')
    grayscale: pictures are stored in grayscale if True, in RGB if False (default 'True')
    asarray: return a np.array. Returns a list if False (default 'True')
    n_img: number of images to be loaded from the path (all by default)

    Returns:
        np.array of n_img pictures
        or list of n_img pictures if asarray = False
    """
    X = []

    if type(n_img) == int:
        i = 0

    if how == 'one':
        for file in os.listdir(path):                              # get every file in the folder
            img = cv2.imread(os.path.join(path, file))                  # load the image
            if grayscale:
                gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
                res = cv2.resize(gray, dsize=size)             # make it RGB (cv2 uses BGR)
                X.append(res)
            else:
                clr = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
                X.append(clr)
            if type(n_img) == int:
                i += 1
                if i == n_img:
                    if asarray:
                        return np.array(X)
                    else:
                        return X

    if how == 'many':
        for folder in os.listdir(path):
            if isdir(os.path.join(path, folder)):
                for file in os.listdir(os.path.join(path, folder)):                              # get every file in the folder
                    img = cv2.imread(os.path.join(path, folder, file))                  # load the image
                    if grayscale:
                        gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)             # make it RGB (cv2 uses BGR)
                        res = cv2.resize(gray, dsize=size)
                        X.append(res)
                    else:
                        clr = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
                        X.append(img)

                    if type(n_img) == int:
                        i += 1
                        if i == n_img:
                            if asarray:
                                return np.array(X)
                            else:
                                return X

    if asarray:
        return np.array(X)
    else:
        return X

picture_file_types = ('.png', '.jpg', '.jpeg', '.tiff', '.bmp', '.gif', '.heic')

def get_image_dict(path, grayscale = True, size = (100, 100), vgg16 = False):
    """loads all images into a dictionary with file names as keys

    Args:
        path (string): the system path from which to load the images
        grayscale (bool, optional): load images as grayscale (True) or in colour (False). Defaults to True.
        size (tuple, optional): Resize images to the size expected by the model. Defaults to (100, 100).

    Returns:
        A dictionary of images as np.arrays with the file names as keys
    """
    #instantiating a dictionary with picture file names as keys
    img_dict = {file:0 for file in os.listdir(path) if file.lower().endswith(picture_file_types)}

    for file in os.listdir(path):
        if file.lower().endswith(picture_file_types):                              #get every image file in folder
            img = cv2.imread(os.path.join(path, file))                  # load the image
            if grayscale:
                gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)            #grayscale
                res = cv2.resize(gray, dsize=size)                    #resize
                img_dict[file] = res
            else:
                if vgg16:
                    clr = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
                    res = cv2.resize(clr, dsize=size)
                    preproc = preprocess_input(res)
                    img_dict[file] = preproc
                else:
                    clr = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
                    res = cv2.resize(gray, dsize=size)
                    img_dict[file] = clr

    return img_dict



import os
import matplotlib.pyplot as plt

def get_directory_size(directory):
    """Returns the `directory` size in bytes."""
    total = 0
    try:
        # print("[+] Getting the size of", directory)
        for entry in os.scandir(directory):
            if entry.is_file():
                # if it's a file, use stat() function
                # fif entry.lower().endswith(('.png', '.jpg', '.jpeg', '.tiff', '.bmp', '.gif')):
                total += entry.stat().st_size
            elif entry.is_dir():
                # if it's a directory, recursively call this function
                total += get_directory_size(entry.path)
    except NotADirectoryError:
        # if `directory` isn't a directory, get the file size then
        return os.path.getsize(directory)
    except PermissionError:
        # if for whatever reason we can't open the folder, return 0
        return 0
    return total
def get_size_format(b, factor=1024, suffix="B"):
    """
    Scale bytes to its proper byte format
    e.g:
        1253656 => '1.20MB'
        1253656678 => '1.17GB'
    """
    for unit in ["", "K", "M", "G", "T", "P", "E", "Z"]:
        if b < factor:
            return f"{b:.2f}{unit}{suffix}"
        b /= factor
    return f"{b:.2f}Y{suffix}"

def plot_pie(sizes, names):
    """Plots a pie where `sizes` is the wedge sizes and `names` """
    plt.pie(sizes, labels=names, autopct=lambda pct: f"{pct:.2f}%")
    plt.title("Different Sub-directory sizes in bytes")
    plt.show()

def get_size(folder_path):

    directory_sizes = []
    names = []
    # iterate over all the directories inside this path
    for directory in os.listdir(folder_path):
        directory = os.path.join(folder_path, directory)
        # get the size of this directory (folder)
        directory_size = get_directory_size(directory)
        if directory_size == 0:
            continue
        directory_sizes.append(directory_size)
        names.append(os.path.basename(directory) + ": " + get_size_format(directory_size))

    #print("[+] Total directory size:", get_size_format(sum(directory_sizes)))
    #plot_pie(directory_sizes, names)
    return get_size_format(sum(directory_sizes))



def image_exif(image_directory):
  """From image directory get the EXIF(information) of the image"""
  from exif import Image
  import os
  # assign directory
  # iterate over files in that directory
  if os.path.isfile(image_directory):
      with open(image_directory, "rb") as test_file:
        new_file = Image(test_file)
        #print(image_directory)

        return new_file

def get_location(img_exif):
    """Get the geolocation of the picture from EXIF"""
    from geopy.geocoders import Nominatim
    #get latitude and longitude
    img_lat = img_exif.gps_latitude[0] + (img_exif.gps_latitude[1] / 60) + (img_exif.gps_latitude[2] / 3600)
    img_lon = img_exif.gps_longitude[0] + (img_exif.gps_longitude[1] / 60) + (img_exif.gps_longitude[2] / 3600)

    # get the reference of the latitude, if South hemisphere becomes negative
    if img_exif.gps_latitude_ref == "S":
        img_lat = -img_lat

    #  get the reference of the longitude, if West becomes negative
    if img_exif.gps_longitude_ref == "W":
        img_lon = -img_lon

    geolocator = Nominatim(user_agent="Photo")

    # get date in case we need to use for something
    img_date = img_exif.datetime_original
    img_time = img_exif.subsec_time_original

    # reverse the geolocation from latitude e longitude give the place
    location = geolocator.reverse("%s,%s" % (img_lat, img_lon), language='en')
    #print(location.raw["address"])

    if "city" in location.raw["address"].keys():
        town = location.raw["address"]["city"]
    elif "town" in location.raw["address"].keys():
        town = location.raw["address"]["town"]
    elif "village" in location.raw["address"].keys():
        town = location.raw["address"]["village"]
    elif "municipality" in location.raw["address"].keys():
        town = location.raw["address"]["municipality"]
    else:
        town = "No Town"

    return f'{town}, {location.raw["address"]["state"]}, {location.raw["address"]["country"]}'


####################### TAGS #############################
def char_fixer(string):
    """
    Find special characters in a string and replace them for normal letters
    """
    a_acc = "àáâãäåæ"
    e_acc = "èéêë"
    i_acc = "ìíîï"
    o_acc = "ðòóôõö"
    u_acc = "ùúûü"

    location = string
    location_list = []
    for char in location:
        if char.lower() in a_acc:
            char = 'a'
        elif char.lower() in e_acc:
            char = "e"
        elif char.lower() in i_acc:
            char = "i"
        elif char.lower() in o_acc:
            char = "o"
        elif char.lower() in u_acc:
            char = "u"
        elif char.lower() in "ç":
            char = "c"
        location_list.append(char)

    return "".join(location_list)


def add_tags(image, user_comment):
    """
    Add the new Geo Tag to the picture in a EXIF variable
    Add Comment to the picture in a EXIF variable
    """
    try:
        get_location(image)
        place = get_location(image)
        image.image_description = char_fixer(place)
    except:
        image.image_description = "no location"
    #print(image.image_description)
    if user_comment != "0":
        image["user_comment"] = char_fixer(user_comment)
        #print(image["user_comment"])

    return image


def save_tags(image_directory, user_comment="0"):
    """"
    Call the image_exif function for the EXIF information
    Call the add_geo_image_description for updating the EXIF of the picture
    Save the image with updated Tags
    """

    #filename = os.path.split(image_directory)[1]
    image_info = image_exif(image_directory)
    img32 = add_tags(image_info, user_comment)
    # if comment == True:
    #     img32 = add_comment_tag(image_info, comment)

    with open(image_directory, 'wb') as image_ex:
        image_ex.write(img32.get_file())
        #print(image_ex.write(img32.get_file()))
        #print("File information updated")



'''Imports'''
from tensorflow.keras import models
from photosorganisation.data import get_image_dict
import os
import cv2
import numpy as np
import shutil
import joblib
import sys
from termcolor import colored
def blurry_classifier(folder_path):
    print(colored("Looking for Blurry Photos...", "cyan", attrs=['bold']))
    loca_path = os.path.dirname(__file__)
    model_path = os.path.join(loca_path, "trained_models", "blur_detector.h5")
    #model_path = "photosorganisation/trained_models/blur_detector.h5"
    pipeline = models.load_model(model_path)

    path = folder_path
    #path = os.path.join('.', 'raw_data', 'test_mantha')

    img_dict = get_image_dict(path, size = (200,200))
    blurry_dump = os.path.join(path,'Blurry')


    if not os.path.exists(blurry_dump):
        os.mkdir(blurry_dump)



    for k in img_dict.keys():
        pred = pipeline.predict(np.expand_dims(img_dict[k], axis = 0))
        if pred > 0.5:
            shutil.move((os.path.join(path, k)), (os.path.join(path, 'Blurry', k)))



'''Imports'''
from tensorflow.keras import models
from photosorganisation.data import get_image_dict
import os
import cv2
import numpy as np
import shutil
import sys
from tensorflow.keras.applications.vgg16 import preprocess_input


def photomulticlassifier(folder_path):
    loca_path = os.path.dirname(__file__)
    model_path = os.path.join(loca_path, "trained_models",
                              "photo_multi_classifier.h5")
    #model_path = "photosorganisation/trained_models/meme_classifier.h5"
    pipeline = models.load_model(model_path)

    path = folder_path
    #path = os.path.join('.', 'raw_data', 'test_mantha')

    img_dict = get_image_dict(path, grayscale=False, size=(150, 150), vgg16=True)
    city_dump = os.path.join(path, 'city')
    nature_dump = os.path.join(path, 'nature')
    food_dump = os.path.join(path, 'food')
    animals_dump = os.path.join(path, 'animals')
    dumps = [city_dump,nature_dump,food_dump,animals_dump]
    for dump in dumps:
        if not os.path.exists(dump):
            os.mkdir(dump)

    class_names = ['city', 'nature', 'food', 'animals']
    for k in img_dict.keys():
       # img_preprocessed = preprocess_input(img_dict[k])
        pred = pipeline.predict(np.expand_dims(img_dict[k], axis=0))
        pred_new = [round(val) for val in pred[0]]
        if 1 in pred_new:
            i=pred_new.index(1)
            shutil.move((os.path.join(path, k)), (os.path.join(path, class_names[i], k)))

#photomulticlassifier('/home/aswathy/code/StorageHeroes/raw_data/test_multiclassifier')



from matplotlib import pyplot
from PIL import Image
from numpy import asarray
import cv2
from mtcnn.mtcnn import MTCNN
import sys
import os
import shutil
#detect faces
def detect_faces(filename):
    img = pyplot.imread(filename)
    #create detector using default weights
    detector = MTCNN()
    #detect all faces in image
    results = detector.detect_faces(img)
    return results


def extract_faces(filename, required_size=(224, 224)):
    #load image from file
    img = pyplot.imread(filename)
    #load detected faces from detector
    results = detect_faces(filename)
    faces_in_image = []
    # extract the bounding box from the faces
    for i in range(len(results)):
        x1, y1, width, height = results[i]['box']
        x2, y2 = x1 + width, y1 + height
        face = img[y1:y2, x1:x2]
        #draw erectangle
        cv2.rectangle(img, (x1, y2), (x2, y1), (0, 0, 255), 3)
        #resize the face pixels
        image = Image.fromarray(face)
        image = image.resize(required_size)
        face_array = asarray(image)
        faces_in_image.append(face_array)
    return faces_in_image, img
#function to decide if there are ppl or not
def people(filename,folder_path):
    people = detect_faces(filename)
    if len(people) > 0:
        people_dump = os.path.join(folder_path, 'People')
        if not os.path.exists(people_dump):
            os.mkdir(people_dump)
        #print(people_dump)
        shutil.move(filename, people_dump)

    else:
        pass
####
if __name__ == '__main__':
    number_of_folders = len(sys.argv)
    dirs_to_scan = sys.argv[1:]
    photos_with_people = 0
    # TO-DO make dict with file names
    for dir in dirs_to_scan:
        # get path, get files
        for file in dir:
            if people(filename):
                photos_with_people + 1



from setuptools import find_packages
from setuptools import setup

with open('requirements.txt') as f:
    content = f.readlines()
requirements = [x.strip() for x in content if 'git+' not in x]

setup(
    name='photosorganisation',
    version="1.0.2.1",
    description=(
                "This python Package helps you sort and organize photos."
                "Duplicates,Memes,Screenshots and Blurry Photos are identifed."
                "Photos are tagged and classified for Geo-Location, Selfies,Animals etc."
    ),
    packages=find_packages(),
    install_requires=requirements,
    test_suite='tests',
    #include_package_data: to install data from MANIFEST.in
    include_package_data=True,
    scripts=[
        'scripts/photosorganisation-run'
    ],
    zip_safe=False)



import cv2
import numpy as np
from photosorganisation.data import load_data, get_data_gcp, get_image_dict
from tensorflow.keras import models
import os
import shutil
import sys
from pathlib import Path
from termcolor import colored
import emoji

def run_screenshot(folder_path):
    print(emoji.emojize(colored(":eyes: Looking for Screenshots :mobile_phone_with_arrow: ...", "cyan", attrs=['bold'])))
    #model = models.load_model('cnn_screenshots.h5')
    loca_path = os.path.dirname(__file__)
    model_path =  os.path.join(loca_path, "trained_models","cnn_screenshots.h5")
    #print(model_path)
    model = models.load_model(model_path)
    #path = os.path.join('.', 'raw_data', 'test_mantha')
    path = folder_path

    img_dict = get_image_dict(path, size = (150,150))
    """     img_dict = {file:0 for file in os.listdir(path) if file.lower().endswith(('.png', '.jpg', '.jpeg', '.tiff', '.bmp', '.gif'))}

    for file in os.listdir(path):
        if file.lower().endswith(('.png', '.jpg', '.jpeg', '.tiff', '.bmp', '.gif')):                                    # get every file in the folder
            img = cv2.imread(os.path.join(path, file))                  # load the image
            gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
            res = cv2.resize(gray, dsize=(150, 150))
            #res = res / 255.
            #res = np.expand_dims(res, axis = -1)             # make it RGB (cv2 uses BGR)
            img_dict[file] = res """

    screenshots_dump = os.path.join(path,'Screenshots')

    if not os.path.exists(screenshots_dump):
        os.mkdir(screenshots_dump)

    #print(img_dict.keys())

    for k in img_dict.keys():
        pred = model.predict(np.expand_dims(img_dict[k], axis=0))
        if pred > 0.5:
            shutil.move((os.path.join(path, k)), (os.path.join(screenshots_dump, k)))














    #memes1 = load_data(path = memes_path_1, how = 'many', asarray = False)
    #print(len(memes1))
    #memes = load_data(path = memes_path_2, how = 'many', n_img = 4500)
    #print(len(memes2))
    #memes = memes1.extend(memes2)
    #print(len(memes))

    #memes = np.array(memes)
    #memes

# not_memes = load_data(nonmemes_path, n_img = 4500)

#print(memes.shape)
#print(not_memes.shape)
